@@ -1,4 +1,4 @@
-#!/usr/bin/env bash
+#! /usr/bin/env bash
 
 # Copyright (C) 2007 Juan Manuel Borges Caño
 
@@ -47,19 +47,27 @@ function h2i_version
 	exit 0
 }
 
-function h2i_result
+function h2i_cat
 {
-	if [[ $? == 0 ]]
-	then echo "done"
-	else echo "error"; exit 1
-	fi
+	sed -e "s,^,$H2I: ," "$1"
+}
+
+function h2i_info
+{
+	echo "Try \`$H2I --help' for more information."
 }
 
 function h2i_error
 {
-	echo "$H2I: $@"
-	echo "Try \`$H2I --help' for more information."
-	exit 1
+	echo "$H2I: $@"; h2i_info; exit 1
+}
+
+function h2i_check_and_echo
+{
+	if [[ $? == 0 ]]
+	then echo "done"
+	else echo "error"; h2i_cat "$1"; h2i_info; exit 1
+	fi
 }
 
 TEMP=$(getopt -o "d:n:a:c:b:g:hv" \
@@ -126,12 +134,10 @@ fi
 #   or:
 function h2i_get_usage
 {
-	O=$IFS;IFS=$'\n'
-	for line in $(egrep -i '^usage:|^  or:' <<< "$1")
+	while IFS=$'\n' read line
 	do
 		echo "$line"
-	done
-	IFS=$O
+	done < <(egrep -i '^usage:|^  or:' <<< "$1")
 }
 H2I_INFO_USAGE=$(h2i_get_usage "$H2I_OUTPUT")
 
@@ -153,12 +159,12 @@ function h2i_get_description
 }
 H2I_INFO_DESCRIPTION=$(h2i_get_description "$H2I_OUTPUT")
 
-# get options (starts after the description with the mandatory line for long and short options and ends with the bugreport comment, last line if present)
-function h2i_get_options
+# get options (starts after the description with the mandatory line for long and short options and ends with the empty line before the bug report address, last line if present)
+function h2i_get_options_and_comments
 {
 	let H2I_START=$(grep -nm 1 '^$' <<< "$1" | cut -d':' -f1)+1
 	H2I_LAST_LINE=$(tail -1 <<< "$1")
-       	if grep -qi 'report' <<< "$H2I_LAST_LINE"
+       	if grep -qi 'report bugs' <<< "$H2I_LAST_LINE"
 	then
 		let H2I_END=$(wc -l <<< "$1")-1
 	else
@@ -166,7 +172,7 @@ function h2i_get_options
 	fi
 	sed -n ${H2I_START},${H2I_END}p <<< "$1"
 }
-H2I_PROGRAM_OPTIONS=$(h2i_get_options "$H2I_OUTPUT")
+H2I_PROGRAM_OPTIONS_AND_COMMENTS=$(h2i_get_options_and_comments "$H2I_OUTPUT")
 
 # convert options to info
 
@@ -181,23 +187,21 @@ function h2i_parse_args
 	done
 	if grep -qe '^--\?\w\+' <<< "$line"
 	then
-		sed -re 's,^([^[:space:]]+)[[:space:]]+.*$,\1,' <<< "$line"
-		line=$(sed -re 's,^[^[:space:]]+[[:space:]]+(.*)$,\1,' <<< "$line")
+		sed -e 's,^\([^[:space:]]\+\)[[:space:]]\+.*$,\1,' <<< "$line"
+		line=$(sed -e 's,^[^[:space:]]\+[[:space:]]\+\(.*\)$,\1,' <<< "$line")
 	else
 		echo
 	fi
 	echo "$line"
 }
 
-function h2i_parse_options
+function h2i_parse_options_and_comments
 {
-	O=$IFS; IFS=$'\n'
-	while read line
+	while IFS=$'\n' read line
 	do
-		IFS=$O
 		if [[ -n $H2I_OPTIONS_ENDED ]]
 		then
-			line=$(sed -e 's,[ \t]\+, ,g' -e 's,^ ,,' <<< "$line")
+			line=$(sed -e 's,[[:space:]]\+, ,g' -e 's,^ ,,' <<< "$line")
 			echo "$line"
 		else
 			if grep -qe '^  -\w\+' <<< "$line"
@@ -222,38 +226,36 @@ function h2i_parse_options
 							if [[ -z $H2I_OPTIONS_ENDED ]]
 							then echo "@end table"; echo; H2I_OPTIONS_ENDED="true"
 							fi
-							if grep -qi "options:" <<< "$line"
+							if grep -qi 'options:' <<< "$line"
 							then
-								line=$(sed -e 's,[ \t]\+, ,g' -e 's,^ ,,' <<< "$line")
+								line=$(sed -e 's,[[:space:]]\+, ,g' -e 's,^ ,,' <<< "$line")
 								echo "$line"; echo; echo "@table @code";
 								unset -v H2I_OPTIONS_ENDED ;
 							else
 								if [[ -z $H2I_OPTIONS_INFO_STARTED ]]
 								then echo "@verbatim"; H2I_OPTIONS_INFO_STARTED="true"
 								fi
-							       	line=$(sed -e 's,[ \t]\+, ,g' -e 's,^ ,,' <<< "$line")
+							       	line=$(sed -e 's,[[:space:]]\+, ,g' -e 's,^ ,,' <<< "$line")
 								echo "$line"
 							fi
 						fi
 					else 
-						line=$(sed -e 's,[ \t]\+, ,g' -e 's,^ ,,' <<< "$line"); echo "$line";
+						line=$(sed -e 's,[[:space:]]\+, ,g' -e 's,^ ,,' <<< "$line"); echo "$line";
 					fi
 				fi
 			fi
 		fi
-		O=$IFS; IFS=$'\n'
 	done <<< "$1"
-	if [[ -n $H2I_OPTIONS_INFO_STARTED ]]; then echo "@end verbatim"; else printf "@end table"; fi
-	IFS=$O
+	if [[ -n $H2I_OPTIONS_INFO_STARTED ]]; then echo "@end verbatim"; else echo "@end table"; fi
 }
-H2I_INFO_OPTIONS=$(h2i_parse_options "$H2I_PROGRAM_OPTIONS")
+H2I_INFO_OPTIONS_AND_COMMENTS=$(h2i_parse_options_and_comments "$H2I_PROGRAM_OPTIONS_AND_COMMENTS")
 
 # convert bugreport addres to info
 function h2i_parse_bugreport()
 {
 	H2I_COMMENT=$(tail -n 1 <<< "$1")
-	if grep -qi "report bugs" <<< "$H2I_COMMENT"
-	then cut -d'<' -f2 <<< "$H2I_COMMENT" | cut -d'>' -f1| sed -re 's,(\w@),\1@,g'
+	if grep -qi 'report bugs' <<< "$H2I_COMMENT"
+	then cut -d'<' -f2 <<< "$H2I_COMMENT" | cut -d'>' -f1 | sed -e 's,\([^@]\+@\),\1@,g'
 	else echo "Insert your bug report address here"
 	fi
 }
@@ -262,124 +264,127 @@ if [[ -z $H2I_INFO_BUGREPORT ]]
 then H2I_INFO_BUGREPORT=$(h2i_parse_bugreport "$H2I_OUTPUT")
 fi
 
+# create the tmp log
+H2I_ERROR_LOG=$(mktemp)
+
 # generate the texi file
 if [[ -a ${H2I_INFO_PROGRAM_NAME}.texi ]]
 then
 	h2i_error "${H2I_INFO_PROGRAM_NAME}.texi: file exists"
 else
 	printf "Creating ${H2I_INFO_PROGRAM_NAME}.texi..."
-	cat > "${H2I_INFO_PROGRAM_NAME}.texi" <<-EOF
-	\input texinfo	@c -*-texinfo-*-
-	@comment It was generated by $H2I $H2I_VERSION.
-	@comment %**start of header
-	@setfilename ${H2I_INFO_PROGRAM_NAME}.info
-	@include version.texi
-	@settitle $H2I_INFO_PROGRAM_NAME @value{VERSION}
-	@syncodeindex pg cp
-	@comment %**end of header
-	@copying
-	This manual is for $H2I_INFO_PROGRAM_NAME
-	(version @value{VERSION}, @value{UPDATED}),
-	which is a program that $H2I_INFO_DESCRIPTION
+		cat 2>> "$H2I_ERROR_LOG" > "${H2I_INFO_PROGRAM_NAME}.texi" <<-EOF
+		\input texinfo	@c -*-texinfo-*-
+		@comment It was generated by $H2I $H2I_VERSION.
+		@comment %**start of header
+		@setfilename ${H2I_INFO_PROGRAM_NAME}.info
+		@include version.texi
+		@settitle $H2I_INFO_PROGRAM_NAME @value{VERSION}
+		@syncodeindex pg cp
+		@comment %**end of header
+		@copying
+		This manual is for $H2I_INFO_PROGRAM_NAME
+		(version @value{VERSION}, @value{UPDATED}),
+		which is a program that $H2I_INFO_DESCRIPTION
 
-	Copyright @copyright{} $H2I_INFO_YEAR $H2I_INFO_COPYRIGHT
+		Copyright @copyright{} $H2I_INFO_YEAR $H2I_INFO_COPYRIGHT
 
-	@quotation
-	Permission is granted to copy, distribute, and/or modify this document
-	under the terms of the GNU Free Documentation License, Version 1.2 or
-	any later version published by the Free Software Foundation; with no
-	Invariant Sections, with the Front-Cover Texts being \`\`A Manual,''
-	and with the Back-Cover Texts as in (a) below. A copy of the
-	license is included in the section entitled \`\`GNU Free Documentation
-	License.''
+		@quotation
+		Permission is granted to copy, distribute, and/or modify this document
+		under the terms of the GNU Free Documentation License, Version 1.2 or
+		any later version published by the Free Software Foundation; with no
+		Invariant Sections, with the Front-Cover Texts being \`\`A Manual,''
+		and with the Back-Cover Texts as in (a) below. A copy of the
+		license is included in the section entitled \`\`GNU Free Documentation
+		License.''
 
-	(a) The Back-Cover Text is: \`\`You have freedom to copy and modify
-	this Manual, like GNU software.''
-	@end quotation
-	@end copying
-	@dircategory $H2I_INFO_DIRCATEGORY
-	@direntry
-	* $H2I_INFO_PROGRAM_NAME: ($H2I_INFO_PROGRAM_NAME). $H2I_INFO_DESCRIPTION
-	@end direntry
+		(a) The Back-Cover Text is: \`\`You have freedom to copy and modify
+		this Manual, like GNU software.''
+		@end quotation
+		@end copying
+		@dircategory $H2I_INFO_DIRCATEGORY
+		@direntry
+		* $H2I_INFO_PROGRAM_NAME: ($H2I_INFO_PROGRAM_NAME). $H2I_INFO_DESCRIPTION
+		@end direntry
 
-	@titlepage
-	@title $H2I_INFO_PROGRAM_NAME
-	@subtitle for version @value{VERSION}, @value{UPDATED}
-	@author $H2I_INFO_AUTHOR (@email{$H2I_INFO_BUGREPORT})
-	@page
-	@vskip 0pt plus 1filll
-	@insertcopying
-	@end titlepage
+		@titlepage
+		@title $H2I_INFO_PROGRAM_NAME
+		@subtitle for version @value{VERSION}, @value{UPDATED}
+		@author $H2I_INFO_AUTHOR (@email{$H2I_INFO_BUGREPORT})
+		@page
+		@vskip 0pt plus 1filll
+		@insertcopying
+		@end titlepage
 
-	@contents
+		@contents
 
-	@ifnottex
-	@node Top
-	@top $H2I_INFO_PROGRAM_NAME
-	@insertcopying
-	@end ifnottex
+		@ifnottex
+		@node Top
+		@top $H2I_INFO_PROGRAM_NAME
+		@insertcopying
+		@end ifnottex
 
-	@menu
-	* Introduction:: Introduction.
-	* Invoking $H2I_INFO_PROGRAM_NAME::	Invocation.
-	* Examples:: Some samples of use.
-	* Reporting Bugs:: Reporting bugs
-	* Copying This Manual:: The license for this documentation.
-	@end menu
+		@menu
+		* Introduction:: Introduction.
+		* Invoking $H2I_INFO_PROGRAM_NAME::	Invocation.
+		* Examples:: Some samples of use.
+		* Reporting Bugs:: Reporting bugs
+		* Copying This Manual:: The license for this documentation.
+		@end menu
 
-	@node Introduction 
-	@chapter Introduction
-	@pindex introduction
-	@cindex introduction
-	$H2I_INFO_DESCRIPTION
+		@node Introduction 
+		@chapter Introduction
+		@pindex introduction
+		@cindex introduction
+		$H2I_INFO_DESCRIPTION
 
-	@node Invoking $H2I_INFO_PROGRAM_NAME
-	@chapter Invoking $H2I_INFO_PROGRAM_NAME
-	@pindex invoking
-	@cindex invoking @command{$H2I_INFO_PROGRAM_NAME}
-	@verbatim
-	$H2I_INFO_USAGE
-	@end verbatim
+		@node Invoking $H2I_INFO_PROGRAM_NAME
+		@chapter Invoking $H2I_INFO_PROGRAM_NAME
+		@pindex invoking
+		@cindex invoking @command{$H2I_INFO_PROGRAM_NAME}
+		@verbatim
+		$H2I_INFO_USAGE
+		@end verbatim
 
-	@command{$H2I_INFO_PROGRAM_NAME} may be invoked with the following command-line options:
+		@command{$H2I_INFO_PROGRAM_NAME} may be invoked with the following command-line options:
 
-	$H2I_INFO_OPTIONS
+		$H2I_INFO_OPTIONS_AND_COMMENTS
 
-	@node Examples
-	@chapter Examples
-	@pindex examples
-	@cindex examples
-	Insert your examples here
+		@node Examples
+		@chapter Examples
+		@pindex examples
+		@cindex examples
+		Insert your examples here
 
-	@node Reporting Bugs
-	@chapter Reporting Bugs
-	@pindex bugs
-	@cindex reporting bugs
-	Email bug reports to @email{$H2I_INFO_BUGREPORT}. Be sure to include the word @dfn{$H2I_INFO_PROGRAM_NAME} somewhere in the @code{Subject:} field. Also, please include the output of @samp{$H2I_INFO_PROGRAM_NAME --version} in the body of your report if at all possible.
+		@node Reporting Bugs
+		@chapter Reporting Bugs
+		@pindex bugs
+		@cindex reporting bugs
+		Email bug reports to @email{$H2I_INFO_BUGREPORT}. Be sure to include the word @dfn{$H2I_INFO_PROGRAM_NAME} somewhere in the @code{Subject:} field. Also, please include the output of @samp{$H2I_INFO_PROGRAM_NAME --version} in the body of your report if at all possible.
 
 
-	@node Copying This Manual
+		@node Copying This Manual
 
-	@appendix Copying This Manual
-	@menu
-	* GNU Free Documentation License::	License for copying this manual.
-	@end menu
+		@appendix Copying This Manual
+		@menu
+		* GNU Free Documentation License::	License for copying this manual.
+		@end menu
 
-	@include fdl.texi
+		@include fdl.texi
 
-	@bye
-	EOF
-	h2i_result
+		@bye
+		EOF
+	h2i_check_and_echo "$H2I_ERROR_LOG"
 fi
 
-# copy the fdl.texi file
+# generate fdl.texi
 if [[ -a fdl.texi ]]
 then
 	h2i_error "fdl.texi: file exists"
 else
-	printf "Copying fdl.texi..."
-	cp "$H2I_DATADIR/fdl.texi" .
-	h2i_result
+	printf "Creating fdl.texi..."
+		cp "$H2I_DATADIR/fdl.texi" . 2>> "$H2I_ERROR_LOG" 
+	h2i_check_and_echo "$H2I_ERROR_LOG"
 fi
 
 # generate version.texi if requested
@@ -391,10 +396,10 @@ then
 	else
 		H2I_INFO_UPDATED=$(date '+%-e %B %Y')
 		printf "Creating version.texi..."
-		cat > "version.texi" <<-EOF
-		@set UPDATED $H2I_INFO_UPDATED
-		@set VERSION $H2I_INFO_VERSION
-		EOF
-		h2i_result
+			cat 2>> "$H2I_ERROR_LOG" > "version.texi" <<-EOF
+			@set UPDATED $H2I_INFO_UPDATED
+			@set VERSION $H2I_INFO_VERSION
+			EOF
+		h2i_check_and_echo "$H2I_ERROR_LOG"
 	fi
 fi
